
/*
  Research carried out within the scope of the Associated International Laboratory: Joint Japanese-French Robotics Laboratory (JRL)
 
  Developed by Mathieu Poirier (LAAS-CNRS)
 
*/


/*****************************************
 INCLUDES
*******************************************/

#include "hppOpenHRP/configSwap.h"

/*****************************************
 DEFINES
*******************************************/

/*****************************************
 METHODS
*******************************************/

// ==========================================================================

void toOpenHRPConfig (CkwsConfigShPtr hppConfig, vector<double> &openHRPConfig) {

  // RLEG
  openHRPConfig[0] = hppConfig->dofValue(6) ;
  openHRPConfig[1] = hppConfig->dofValue(7) ;
  openHRPConfig[2] = hppConfig->dofValue(8) ;
  openHRPConfig[3] = hppConfig->dofValue(9) ;
  openHRPConfig[4] = hppConfig->dofValue(10) ;
  openHRPConfig[5] = hppConfig->dofValue(11) ;
  
  // LLEG
  openHRPConfig[6] = hppConfig->dofValue(12) ;
  openHRPConfig[7] = hppConfig->dofValue(13) ;
  openHRPConfig[8] = hppConfig->dofValue(14) ;
  openHRPConfig[9] = hppConfig->dofValue(15) ;
  openHRPConfig[10] = hppConfig->dofValue(16) ;
  openHRPConfig[11] = hppConfig->dofValue(17) ;
  
  // CHEST
  openHRPConfig[12] = hppConfig->dofValue(18) ;
  openHRPConfig[13] = hppConfig->dofValue(19) ;
  
  // HEAD
  openHRPConfig[14] = hppConfig->dofValue(20) ;
  openHRPConfig[15] = hppConfig->dofValue(21) ;
  
  // RARM
  openHRPConfig[16] = hppConfig->dofValue(22) ;
  openHRPConfig[17] = hppConfig->dofValue(23) ;
  openHRPConfig[18] = hppConfig->dofValue(24) ;
  openHRPConfig[19] = hppConfig->dofValue(25) ;
  openHRPConfig[20] = hppConfig->dofValue(26) ;
  openHRPConfig[21] = hppConfig->dofValue(27) ;
  openHRPConfig[22] = hppConfig->dofValue(28) ;
  
  // RHAND
  openHRPConfig[30] = hppConfig->dofValue(29) ;
  openHRPConfig[31] = hppConfig->dofValue(30) ;
  openHRPConfig[32] = hppConfig->dofValue(31) ;
  openHRPConfig[33] = hppConfig->dofValue(32) ;
  openHRPConfig[34] = hppConfig->dofValue(33) ;
  
  
  // LARM
  openHRPConfig[23] = hppConfig->dofValue(34) ;
  openHRPConfig[24] = hppConfig->dofValue(35) ;
  openHRPConfig[25] = hppConfig->dofValue(36) ;
  openHRPConfig[26] = hppConfig->dofValue(37) ;
  openHRPConfig[27] = hppConfig->dofValue(38) ;
  openHRPConfig[28] = hppConfig->dofValue(39) ;
  openHRPConfig[29] = hppConfig->dofValue(40) ;
  
  // LHAND
  openHRPConfig[35] = hppConfig->dofValue(41) ;
  openHRPConfig[36] = hppConfig->dofValue(42) ;
  openHRPConfig[37] = hppConfig->dofValue(43) ;
  openHRPConfig[38] = hppConfig->dofValue(44) ;
  openHRPConfig[39] = hppConfig->dofValue(45) ;
}

// ==========================================================================

void toHppRotation(const double inR, const double inP, const double inY,
		   double &outX, double &outY, double &outZ)
{
    const double cr=cos(inR),sr=sin(inR),cp=cos(inP),sp=sin(inP),cy=cos(inY),sy=sin(inY);
    double R[3][3];
    R[0][0] = cp*cy;
    R[0][1] = sr*sp*cy - cr*sy;
    R[0][2] = cr*sp*cy + sr*sy;
    R[1][0] = cp*sy;
    R[1][1] = sr*sp*sy + cr*cy;
    R[1][2] = cr*sp*sy - sr*cy;
    R[2][0] = -sp;
    R[2][1] = sr*cp;
    R[2][2] = cr*cp;

    double      cos_y;
    double      tr_x;
    double      tr_y;

    // make sure all values are in [-1,1]
    // as trigonometric functions would
    // fail when given values such as 1.00000001
    for(unsigned int i=0; i<3; i++)
	{
	    for(unsigned int j=0; j<3; j++)
		{
		    if(R[i][j] < -1.)
			{
			    R[i][j] = -1;
			}
		    else if(R[i][j] > 1.)
			{
			    R[i][j] = 1;
			}
		}
	}

    outY = asin(R[0][2]);

    cos_y = cos(outY);

    if(fabs(cos_y) > .0005)
	{
	    tr_x =  R[2][2] / cos_y;
	    tr_y = -R[1][2] / cos_y;
	    
	    outX = atan2(tr_y, tr_x);
	    
	    tr_x =  R[0][0] / cos_y;
	    tr_y = -R[0][1] / cos_y;
	}
    else
	{
	    outX = 0.;
	    
	    tr_x = R[1][1];
	    tr_y = R[1][0];
	}
    
    outZ = atan2(tr_y, tr_x);
}

void toHppConfig (vector<double> openHRPConfig, vector<double> waistPosition, CkwsConfigShPtr &hppConfig ) {
  
   // WAIST Position and Orientation
    hppConfig->dofValue(0, waistPosition[0] );   // X
    hppConfig->dofValue(1, waistPosition[1] );   // Y
    hppConfig->dofValue(2, waistPosition[2] );   // Z
    double rx, ry, rz;
    toHppRotation(waistPosition[3], waistPosition[4], waistPosition[5], 
		  rx, ry, rz);
    hppConfig->dofValue(3, rx );   // Rx // rad
    hppConfig->dofValue(4, ry );   // Ry // rad
    hppConfig->dofValue(5, rz );   // RZ // rad

    // RLEG
    hppConfig->dofValue(6, openHRPConfig[0]) ;
    hppConfig->dofValue(7, openHRPConfig[1]) ;
    hppConfig->dofValue(8, openHRPConfig[2]) ;
    hppConfig->dofValue(9, openHRPConfig[3]) ;
    hppConfig->dofValue(10, openHRPConfig[4]) ;
    hppConfig->dofValue(11, openHRPConfig[5]) ;

    // LLEG
    hppConfig->dofValue(12, openHRPConfig[6]) ;
    hppConfig->dofValue(13, openHRPConfig[7]) ;
    hppConfig->dofValue(14, openHRPConfig[8]) ;
    hppConfig->dofValue(15, openHRPConfig[9]) ;
    hppConfig->dofValue(16, openHRPConfig[10]) ;
    hppConfig->dofValue(17, openHRPConfig[11]) ;

    // CHEST
    hppConfig->dofValue(18, openHRPConfig[12]) ;
    hppConfig->dofValue(19, openHRPConfig[13]) ;

    // HEAD
    hppConfig->dofValue(20, openHRPConfig[14]) ;
    hppConfig->dofValue(21, openHRPConfig[15]) ;

    // RARM
    hppConfig->dofValue(22, openHRPConfig[16]) ;
    hppConfig->dofValue(23, openHRPConfig[17]) ;
    hppConfig->dofValue(24, openHRPConfig[18]) ;
    hppConfig->dofValue(25, openHRPConfig[19]) ;
    hppConfig->dofValue(26, openHRPConfig[20]) ;
    hppConfig->dofValue(27, openHRPConfig[21]) ;
    hppConfig->dofValue(28, openHRPConfig[22]) ;

    // RHAND
    hppConfig->dofValue(29, openHRPConfig[30]) ;
    hppConfig->dofValue(30, openHRPConfig[31]) ;
    hppConfig->dofValue(31, openHRPConfig[32]) ;
    hppConfig->dofValue(32, openHRPConfig[33]) ;
    hppConfig->dofValue(33, openHRPConfig[34]) ;


    // LARM
    hppConfig->dofValue(34, openHRPConfig[23]) ;
    hppConfig->dofValue(35, openHRPConfig[24]) ;
    hppConfig->dofValue(36, openHRPConfig[25]) ;
    hppConfig->dofValue(37, openHRPConfig[26]) ;
    hppConfig->dofValue(38, openHRPConfig[27]) ;
    hppConfig->dofValue(39, openHRPConfig[28]) ;
    hppConfig->dofValue(40, openHRPConfig[29]) ;

    // LHAND
    hppConfig->dofValue(41, openHRPConfig[35]) ;
    hppConfig->dofValue(42, openHRPConfig[36]) ;
    hppConfig->dofValue(43, openHRPConfig[37]) ;
    hppConfig->dofValue(44, openHRPConfig[38]) ;
    hppConfig->dofValue(45, openHRPConfig[39]) ;
 
}
